package org.foxarmy.barcodescannerforemployees

import android.content.Context
import android.database.DatabaseUtils
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.provider.BaseColumns
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import java.io.File

object AbstractProductContract {
    object AbstractProductEntry : BaseColumns {
        const val TABLE_NAME = "abstract_products"
        const val BARCODE = "barcode"
        const val PRODUCT_NAME = "name"
        const val PRODUCT_NET_WEIGHT = "net_weight"
        const val IMAGE_FILENAME = "image_filename"
        const val CATEGORY = "category"
    }
}

object CategoriesContract {
    object CategoryEntry : BaseColumns {
        const val TABLE_NAME = "categories"
        const val CATEGORY_NAME = "category_name"
    }
}

object ProductContract {
    object ProductEntry : BaseColumns {
        const val TABLE_NAME = "products"
        const val ABSTRACT_PRODUCT_ID = "abstract_product_id"
        const val AMOUNT = "amount"
        const val DATE_OF_PRODUCTION = "date_of_production"
        const val EXPIRY_DATE = "expiry_date"
    }
}

const val SQL_CREATE_ABSTRACT_PRODUCTS_TABLE =
    "CREATE TABLE ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${AbstractProductContract.AbstractProductEntry.BARCODE} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT} REAL," +
            "${AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.CATEGORY} INTEGER)"

const val SQL_CREATE_PRODUCTS_TABLE =
    "CREATE TABLE ${ProductContract.ProductEntry.TABLE_NAME} (" +
        "${BaseColumns._ID} INTEGER PRIMARY KEY," +
        "${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID} INTEGER," +
        "${ProductContract.ProductEntry.AMOUNT} INTEGER," +
        "${ProductContract.ProductEntry.DATE_OF_PRODUCTION} DATE," +
        "${ProductContract.ProductEntry.EXPIRY_DATE} DATE)"

const val SQL_CREATE_CATEGORIES_TABLE =
    "CREATE TABLE ${CategoriesContract.CategoryEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${CategoriesContract.CategoryEntry.CATEGORY_NAME} TEXT)"

class DBStorageController(context: Context) : SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {

    override fun onCreate(db: SQLiteDatabase) {
        db.execSQL(SQL_CREATE_ABSTRACT_PRODUCTS_TABLE)
        db.execSQL(SQL_CREATE_PRODUCTS_TABLE)
        db.execSQL(SQL_CREATE_CATEGORIES_TABLE)
    }

    override fun onUpgrade(db: SQLiteDatabase?, oldV: Int, newV: Int) {
        TODO("Not yet implemented")
    }

    fun eraseCategory (db: SQLiteDatabase, id: Int, context: Context) {
        val productsInCategory = getAllAbstractProductInCategory(db, id)

        for (product in productsInCategory.iterator()) {
            eraseAbstractProduct(db, product.id, context)
        }

        db.delete(CategoriesContract.CategoryEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun getCategoryNameById(db: SQLiteDatabase, id: Int) : String {
        var result = ""

        val projection = arrayOf(CategoriesContract.CategoryEntry.CATEGORY_NAME)
        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(CategoriesContract.CategoryEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, null)

        with (cursor) {
            while (moveToNext()) {
                result = getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME))
            }
        }

        return result
    }

    fun getAllAbstractProductInCategory(db: SQLiteDatabase, id: Int) : List<AbstractProduct> {

        var result = mutableListOf<AbstractProduct>()
        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
        )

        val selection = "${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(AbstractProductContract.AbstractProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, null)

        with(cursor) {
            while (moveToNext()) {
                val abstractProductId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val abstractProductBarcode = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val abstractProductName = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val abstractProductNetWeight = getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val abstractProductImageHash = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))

                val abstractProduct = AbstractProduct(abstractProductId, abstractProductBarcode, abstractProductName, abstractProductNetWeight, abstractProductImageHash, category = id)

                result.add(abstractProduct)
            }
        }

        return result
    }

    fun getAmountOfAbstractProductsInCategory(db:SQLiteDatabase, id: Int) : Int {
        return DatabaseUtils.longForQuery(db, "SELECT COUNT(*) FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?", arrayOf(id.toString())).toInt()
    }
    fun eraseAbstractProduct(db: SQLiteDatabase, id: Int, context: Context) {
        val projection = arrayOf(
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME
        )

        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())
        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )
        var imageHash: String = ""

        with (cursor) {
            while(moveToNext()) {
                val productImageHash = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                imageHash = productImageHash
            }
        }
        val picturesDir = File(context.filesDir, "pictures")
        val thumbnailsDir = File(context.cacheDir, "thumbnails")

        File(picturesDir, "$imageHash.png").delete()
        File(thumbnailsDir, "$imageHash.webp").delete()
        db.delete(AbstractProductContract.AbstractProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    companion object {
        const val DATABASE_VERSION = 1
        const val DATABASE_NAME = "database.db"
    }
}